<?php
namespace Teknisa\DC\Controllers;

use Teknisa\DC\Util\Utilities;
use Teknisa\DC\Util\Messages;
use Teknisa\DC\Services;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\Log\Exception;
use Zeedhi\DevOps\Util\Utilities as DevOpsUtilities;

class Resource {
    /** @var \Service\Resource $resourceService*/
	protected $resourceService;
	protected $resourceProjectService;
	protected $localeService;
	protected $environment;

	public function __construct(Services\Resource $resourceService, Services\ResourceProject $resourceProjectService,
                                Services\Locale $localeService, Utilities $environment) {
		$this->resourceService = $resourceService;
    	$this->resourceProjectService = $resourceProjectService;
    	$this->localeService = $localeService;
		$this->environment = $environment;
	}

    public function findUserData(Request\Filter $request, Response $response) {
        $userData = $this->resourceService->getUserData();
        $response->addDataSet(new DataSet('findUserData', $userData));
    }

    public function saveEditedUserData(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];
        $name = $param['NAME'];
        $email = $param['EMAIL'];
        $photo = empty($param['PHOTO_VIEW']) ? null : $param['PHOTO_VIEW'];
        try {
            $this->resourceService->saveEditedUserDataValidate($name, $email, $photo);
            $userData = $this->resourceService->getUserData();
            $response->addDataSet(new DataSet('userData', $userData));
            $response->addNotification(new Response\Notification("Perfil alterado com sucesso"));
        } catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function changePassword(Request\DataSet $request, Response $response) {
        $dataSet = $request->getDataSet()->getRows();
        $param = $dataSet[0];

        $newPassword = $param['NEW_PASSWORD'];
        $currentPassword = empty($dataSet[0]['PASSWORD']) ? null : $param['PASSWORD'];
        $resourceId = empty($dataSet[0]['RESOURCE_ID']) ? null : $param['RESOURCE_ID'];
        $result = empty($currentPassword)
            ? $this->resourceService->resetResourcePassword($newPassword, $resourceId)
            : $this->resourceService->changePassword($currentPassword, $newPassword);

        if($result) {
            $response->addNotification(new Response\Notification("Senha alterada com sucesso"));
        } else {
            $response->addMessage(new Response\Message("A senha atual informada está incorreta."));
        }
    }

    public function sponsorCountry( Request\Filter $request,Response $response){
        $response->addDataset(new DataSet('country', $this->localeService->getAllCountries()));
    }

    public function findSponsorWithCrud(Request\Filter $request, Response $response){
        try {
            $conditions = $request->getFilterCriteria()->getConditions();
            $page       = $request->getFilterCriteria()->getPage();
            $pageSize   = $request->getFilterCriteria()->getPageSize();
            $sponsor    = $this->resourceService->findSponsorWithCrud($conditions, $page, $pageSize);

            $response->addDataset(new Dataset('sponsorWithCrud', $sponsor));
        } catch(\Exception $e){
            $response->addMessage(new Response\Message('Erro inesperado: ' . $e->getMessage(), Response::STATUS_ERROR));
        }
    }

    public function sponsorProjectData(Request\Filter $request,Response $response)
    {
        $filters = $request->getFilterCriteria()->getConditions();
        $sponsorId = DevOpsUtilities::getValueInFilterDataSourceByName($filters, 'SPONSOR', true);
        $conditions = DevOpsUtilities::createQueryFiltered($filters);
        $dataSetBasic = $this->resourceService->sponsorProjectData($sponsorId, $conditions);
        DevOpsUtilities::setDataSetDefault($request, $response, $dataSetBasic);
    }

    public function basicSponsor(Request\Filter $request,Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $resource = $param[0]["value"];
        $dataSetBasic = array();
        if($resource){
            $dataSetBasic = $this->resourceService->basicSponsor($resource);
        }
        $response->addDataset(new DataSet('basicSponsor', $dataSetBasic));
    }

    public function deleteSponsor(Request\Row $request,Response $response){
        $row = $request->getRow();
        try{
            $this->resourceService->deleteSponsor($row['ID']);
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function findContacts(Request\Filter $request,Response $response){
        $param = $request->getFilterCriteria()->getConditions();
        $resource = Utilities::getValueInFilterDataSourceByName($param, "sponsor", true);
        $filters = DevOpsUtilities::createQueryFiltered($param);
        $dataSetContact = $this->resourceService->formContactList($resource, $filters);
        $response->addDataset(new DataSet('contactList', $dataSetContact));
    }

    public function saveContactSponsorList(Request\DataSet $request,Response $response) {
        try {
            $dataSet=$request->getDataSet()->getRows();
            $param = $dataSet[0];

            $sponsorId = empty($param['SPONSOR'])? null:$param['SPONSOR'];
            $address = empty($param['ADDRESS'])? null:$param['ADDRESS'];
            $city    = empty($param['CITY'])? null:$param['CITY'];
            $district= empty($param['DISTRICT'])? null:$param['DISTRICT'];
            $email   = empty($param['EMAIL'])? null:$param['EMAIL'];
            $name    = empty($param['NAME'])? null:$param['NAME'];
            $phone   = empty($param['PHONE'])? null:$param['PHONE'];
            $status  = empty($param['STATUS']) ? null : $param['STATUS'];
            $zipCode = empty($param['ZIP_CODE'])? null:$param['ZIP_CODE'];
            if(!empty($param['CONTACT'])){
                $contact = $param['CONTACT'];
                $contactId = $this->resourceService->updateContact($contact, $sponsorId, $address, $city, $district, $email, $name, $phone, $status, $zipCode);
            } else {
                $contactId = $this->resourceService->createNewContact($sponsorId, $address, $city, $district, $email, $name, $phone, $status, $zipCode);
            }
            $response->addDataset(new DataSet("contact", $contactId));
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_SAVED));
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteContact(Request\DataSet $request,Response $response){
        $dataSet = $request->getDataSet()->getRows();
        try{
            foreach($dataSet as $param){
                $sponsor = $this->resourceService->deleteSponsor($param['CONTACT']);
            }
            $response->addNotification(new Response\Notification(Messages::SUC_SUCCESSFUL_REMOVED));
        }catch (\Exception $e){
            $response->addMessage(new Response\Message($e->getMessage()));
        }
    }

    public function saveSponsor(Request\DataSet $request,Response $response){
        try{
            $dataSet = $request->getDataSet()->getRows();
            $param = $dataSet[0];
            $login = empty($param['LOGIN'])? null:$param['LOGIN'];
            $password = empty($param['PASSWORD'])? null:$param['PASSWORD'] ;
            $status = empty($param['STATUS'])? 'A':$param['STATUS'] ;
            $structure = null;
            $name = empty($param['NAME'])? null:$param['NAME'];
            $nickName = empty($param['NICK_NAME'])? null:$param['NICK_NAME'];
            $documentNumber = empty($param['DOCUMENT_NUMBER'])? null:$param['DOCUMENT_NUMBER'];
            $description = empty($param['DESCRIPTION'])? null:$param['DESCRIPTION'];

            if(!empty($param['ID'])){
                $resourceId = $param['ID'];
                $resourceId = $this->resourceService->editSponsor($login, $password, $status, $name, $nickName, $documentNumber, $description, $resourceId);
                $message = "Alterações salvas com sucesso";
                $response->addDataSet(new DataSet('ResourceInfo', array($resourceId)));
            }else{
                $resource = $this->resourceService->newSponsor($login, $password, $status, $name, $nickName, $documentNumber, $description);
                $message = "Patrocinador criado!";
                $response->addDataSet(new DataSet('ResourceInfo', array($resource->getId())));
            }
            $response->addNotification(new Response\Notification($message));
        }catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function sponsorState( Request\Filter $request,Response $response){

        $param = $request->getFilterCriteria()->getConditions();
        $countryId = empty($param[0]) ? null : $param[0]['value'];
        if($countryId){
            $dataSet =$this->localeService->getStatesByCountry($countryId);
        }else{
            $dataSet = array();
        }
        $response->addDataset(new DataSet('state', $dataSet));
    }

    public function sponsorCity( Request\Filter $request,Response $response){

        $param = $request->getFilterCriteria()->getConditions();
        $state = empty($param[0]) ? null : $param[0]['value'];
        if($state){
            $dataSet = $this->localeService->getCitiesByState($state);
        }else{
            $dataSet = array();
        }
        $response->addDataset(new DataSet('city', $dataSet));
    }

    public function uploadProfilePicture(Request\DataSet $request, Response $response)
    {
        try{
            $row = $request->getDataSet()->getRows()[0];
            $imagePath = empty($row['IMAGE'][0]['path'])? null :$row['IMAGE'][0]['path'];
            $imagePath = $this->resourceService->uploadProfilePicture($imagePath);
            $response->addDataSet(new DataSet('uploadProfilePicture', $imagePath));
        } catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }

    public function deleteProfilePicture(Request\DataSet $request, Response $response)
    {
        try{
            $this->resourceService->deleteProfilePicture();
        }catch (\Exception $e){
            $response->setError(new Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}