<?php

namespace Teknisa\DC\Controllers;

use Teknisa\DC\Util\Messages;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Controller\Exception;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;

class ResourceProject extends \Zeedhi\Framework\Controller\Simple
{
    protected $resourceProjectService;

    public function __construct(\Teknisa\DC\Services\ResourceProject $resourceProjectService)
    {
        $this->resourceProjectService = $resourceProjectService;
    }

    public function bringProjectsPaths(Request\DataSet $request, Response $response)
    {
        $param = $request->getDataSet()->getRows();
        $user = $param[0]['USER'];
        $projectPaths = $this->resourceProjectService->getAllResourceProjectsPaths($user);
        $response->addDataSet(new DataSet('projectsPaths', $projectPaths));
    }

    public function deleteDirectories(Request\DataSet $request, Response $response){
        $count = 0;
        $dataSet = $request->getDataSet()->getRows();
        foreach ($dataSet as $param){
            try{
                $this->resourceProjectService->deleteDirectory($param['PATH_ID']);
            }catch(\Exception $ex){
                $response->addMessage(new Response\Message(Messages::ERR_NOT_SUCCESSFUL_DELETED_EXPLANATION . $ex->getMessage()));
                $count--;
            }
            $count++;
        }
        $response->addNotification(new Response\Notification("$count diretório(s) deletado(s)."));
    }

    public function dataProjects(Request\Filter $request, Response $response)
    {
        $resourceProjectPaths = $this->resourceProjectService->getAllResourceProjectsPaths();
        $response->addDataSet(new DataSet('userDataProjects', $resourceProjectPaths));
    }

    public function bringResourceProjectsPaths(Request\Filter $request, Response $response) {
        $param = $request->getFilterCriteria()->getConditions();
        if(!empty($param[0]['value'])){
            $projectId = $param[0]['value'];
        }else{
            $projectId = null;
        }
        $resourceProjectPaths = $this->resourceProjectService->getAllResourcePaths($projectId);
        $response->addDataSet(new DataSet('resourceProjectsPaths', $resourceProjectPaths));
    }

    public function saveDirectory(Request\DataSet $request, Response $response) {
        $param = $request->getDataSet()->getRows()[0];
        $replicateConnections = $param['REPLICATE_CONNECTIONS'];
        $projectId = $param['PROJECT_ID'];
        $repository = $param['REPOSITORY_ID'];
        $directory = $param['PATH'];
        $linuxDirectory = $param['LINUX_DIRECTORY'];
        if(isset($param['PATH_ID'])){
            $directoryId = $param['PATH_ID'];
        }else{
            $directoryId = null;
        }
        $resourceProject = $this->resourceProjectService->saveDirectory( $repository, $directory, null, $directoryId,$linuxDirectory,$projectId, $replicateConnections);
        $response->addDataSet(new DataSet('resourceProjectsPaths', array($resourceProject->getId())));
        $response->addNotification(new Response\Notification('Diretório salvo com sucesso !'));
    }

    public function copyProjectPaths(Request\DataSet $request, Response $response) {
        $params = $request->getDataSet()->getRows()[0];
        $selectedPaths = $params['selectedPaths'];
        $project = $params['projectReceiver'];
        $this->resourceProjectService->copyDirectories($selectedPaths, $project);
    }

    public function verifyProjectPaths(Request\DataSet $request, Response $response) {
        $params = $request->getDataSet()->getRows()[0];
        $selectedPaths = $params['selectedPaths'];
        $project = $params['projectReceiver'];
        list($successfulProjectPaths, $failedProjectsPaths) = $this->resourceProjectService->verifyProjectPaths($selectedPaths, $project);
        $response->addDataSet(new DataSet('verifyProjectPaths', array('failedProjectPaths' => $failedProjectsPaths,
            'successfulProjectPaths' => $successfulProjectPaths)));
    }

}
