<?php

namespace Teknisa\DC\Services;

use Zeedhi\Framework\Controller\Exception;
use Teknisa\DC\Util\Utilities;
use Teknisa\DC\Factories;
use Zeedhi\Framework\Session\Session;
use Zeedhi\DevOps\Model\Projectmanagement\Entities;
use Zeedhi\DevOps\Model\Projectmanagement\Repositories;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\Remote\FileUploader\Server;

class Resource
{
    const WORK_RESOURCE    = Entities\Resource::WORK_RESOURCE;
    const CONTACT_RESOURCE = Entities\Resource::CONTACT_RESOURCE;

    const CLASS_SPONSOR = Entities\Resource::CLASS_SPONSOR;
    const CLASS_CONTACT = Entities\Resource::CLASS_CONTACT;

    const IS_USER_TRUE  = Entities\Resource::IS_USER_TRUE;
    const IS_USER_FALSE = Entities\Resource::IS_USER_FALSE;

    const IS_ACTIVE = Entities\Resource::IS_ACTIVE;
    const TEKNISA_ENTITY = '\Teknisa\Model\Geral\Entities\\';

    protected $entityManager;
    protected $environment;
    protected $resourceFactory;
    /** @var Repositories\Resource $resourceRepository */
    protected $resourceRepository;

    /** @var  Task $taskService */
    protected $taskService;
    /** @var Repositories\ResourceProject $resourceProjectRepository */
    protected $resourceProjectRepository;
    /** @var Repositories\City $cityRepository */
    protected $cityRepository;

    private $fileServerFilesUrl;
    private $zhFileUploader;

    public function __construct(EntityManager $entityManager, Utilities $environment,
                                Factories\Resource $resourceFactory, $attachmentsBaseDirectory,
                                $fileServerFilesUrl, Server $zhFileUploader)
    {
        $this->entityManager   = $entityManager;
        $this->environment     = $environment;
        $this->resourceFactory = $resourceFactory;

        $this->cityRepository            = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'City');
        $this->resourceRepository        = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'Resource');
        $this->resourceProjectRepository = $this->entityManager->getRepository(Utilities::ENTITY_DIR . 'ResourceProject');

        $this->profileBaseDirectory = $attachmentsBaseDirectory . DIRECTORY_SEPARATOR . 'profile';

        $this->fileServerFilesUrl = $fileServerFilesUrl;
        $this->zhFileUploader = $zhFileUploader;
    }

    public function getUserData()
    {
        $user = $this->environment->getCurrentUser();
        $resourceId = $user->getId();
        $userData = array(
            "NAME" => $user->getName(),
            "EMAIL" => $user->getEmail(),
        );
        $operatorRepository = $this->entityManager->getRepository(self::TEKNISA_ENTITY . 'Operador');
        $operator = $operatorRepository->findOneBy(array('cdoperador' =>  $user->getExternalCode()));

        if($operator->getDsimagemoper()){
            $userData['IMAGE'] = $this->fileServerFilesUrl . $operator->getDsimagemoper();
        } else {
            $userData['IMAGE'] = null;
        }
        $userData['HAS_IMAGE'] = $userData['IMAGE'];
        return array($userData);
    }

    public function saveEditedUserDataValidate($name, $email, $photo = null) {
        if( ! Utilities::validateNameLogin($name)){
            throw new \Exception("Nome inválido.");
        }

        if( ! Utilities::validateEmail($email)){
            throw new \Exception("Email inválido.");
        }

        $this->saveEditedUserData($name, $email, $photo);
    }

    public function saveEditedUserData($name, $email, $photo = null) {
        $login = $this->environment->getCurrentUser()->getEmail();
        $user = $this->resourceRepository->findOneBy(array(
            "email" => $login,
            "organization" => $this->environment->getCurrentOrganizationId(),
            "status" => self::IS_ACTIVE
        ));

        if ($user) {
            return $this->updateResourceTypeUser($user, $email, $user->getPassword(), $user->getStatus(), $name, $photo);
        }
        return false;
    }

    public function updateResourceTypeUser($resource, $email, $password, $status, $name, $photo = null, $passwordReset= null)
    {
        if(!$passwordReset && ! $this->isEmailAvailable($email)){
            throw new \Exception("O e-mail informado já foi cadastrado para outro usuário.");
        }

        $resourceEntityClass = "\\Zeedhi\\DevOps\\Model\\Projectmanagement\\Entities\\Resource";
        if( ! ($resource instanceof $resourceEntityClass)){
            $resource = $this->entityManager->find($resourceEntityClass, $resource);
        }

        $resourceEntity = $this->resourceFactory->updateResourceTypeUser($resource, $email, $password, $status, $name);
        $this->entityManager->persist($resourceEntity);
        $this->entityManager->flush();

        if( ! empty($photo)){
            $this->uploadPicture($photo, $this->environment->getCurrentUserId());
        }

        return $resourceEntity;
    }

    private function isEmailAvailable($email)
    {
        $resourceEntityClass = "\\Zeedhi\\DevOps\\Model\\Projectmanagement\\Entities\\Resource";
        $usersUsingEmail = $this->entityManager->getRepository($resourceEntityClass)->findBy(array(
            "email" => $email,
            "organization" => $this->environment->getCurrentOrganizationId()
        ));

        if(empty($usersUsingEmail)){
            return true;
        }

        /** @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $user */
        foreach($usersUsingEmail as $user){
            if($user->getId() != $this->environment->getCurrentUserId()){
                return false;
            }
        }

        return true;
    }

    private function uploadPicture($photo, $fileName) {
        $dir = $this->profileBaseDirectory;
        if( ! file_exists($dir)){
            mkdir($dir);
        }

        $dir = $dir . DIRECTORY_SEPARATOR;
        file_put_contents($dir . $fileName, $photo);
    }

    /**
     *
     * This function resets a specified resource's password. Is not needed the current
     * password so, technically, only managers can use this option. It is an alternative
     * for "I forgot my password".
     *
     * @param $newPassword
     * @param $resourceId
     * @return boolean
     */
    public function resetResourcePassword($newPassword, $resourceId) {
        $email = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $resourceId)->getEmail();
        $user = $this->resourceRepository->findOneBy(array(
            "id" => $resourceId,
            "email" => $email,
            "organization" => $this->environment->getCurrentOrganizationId(),
            "status" => "A"
        ));
        if ($user) {
            $this->updateResourceTypeUser($user, $email, (md5($newPassword)), $user->getStatus(), $user->getName(), null, true);
            return true;
        }
        return false;
    }

    public function changePassword($currentPassword, $newPassword) {
        $email = $this->environment->getCurrentUser()->getEmail();
        $userRepository = $this->resourceRepository;
        $user = $userRepository->findOneBy(array(
            "email" => $email,
            "organization" => $this->environment->getCurrentOrganizationId(),
            "password" => (md5($currentPassword)),
            "status" => "A"
        ));
        if ($user) {
            $this->updateResourceTypeUser($user, $user->getEmail(), (md5($newPassword)), $user->getStatus(), $user->getName());
            return true;
        }
        return false;
    }

    public function findSponsorWithCrud($conditions, $page, $pageSize) {
        $filters = Utilities::createQueryFiltered($conditions);
        $organizationId = $this->environment->getCurrentOrganizationId();
        $limit = $page * $pageSize;
        $offset = ($page - 1) * $pageSize;
        return $this->resourceRepository->findResourceSponsor($organizationId, $filters, $limit, $offset);
    }

    public function sponsorProjectData($sponsorId, $conditions)
    {
        $organization = $this->environment->getCurrentOrganizationId();
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Repositories\Resource $repository
         */
        $repository = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource');
        $allProjects = $repository->getSponsorProjectData($sponsorId, $organization, $conditions);
        $allProjectsIds = array();
        foreach ($allProjects as $project) {
            array_push($allProjectsIds, $project['ID']);
        }
        foreach ($allProjects as &$project) {
            if (!in_array($project['PARENT'], $allProjectsIds)) {
                $project['PARENT'] = null;
            }
        }
        return $allProjects;
    }

    public function basicSponsor($resource)
    {
        return $this->resourceRepository->basicSponsor($resource);
    }

    public function formContactList($resource, $filters)
    {
        $organization = $this->environment->getCurrentOrganizationId();
        return $this->resourceRepository->formContactList($resource, $organization, $filters);
    }

    public function updateContact($contactId, $sponsorId, $address, $city, $district, $email, $name, $phone,
                                  $status, $zipCode) {
        try{
            if ($email) {
                $isContact = $this->resourceRepository->findOneBy(array('email' => $email));
                if(!empty($isContact) && $isContact->getId() != $contactId){
                    throw new \Exception("Email já cadastrado. Favor informar outro email.");
                }
            }

            $contact = $this->resourceRepository->find($contactId);
            $sponsor = $this->resourceRepository->find($sponsorId);
            $contact = $this->updateResource($contact, self::CONTACT_RESOURCE, self::IS_USER_FALSE, $status, $email, $name,
                null, self::CLASS_CONTACT, null, null, $phone, $address, $district, $city, $zipCode, $sponsor);
            $this->entityManager->persist($contact);
            $this->entityManager->flush();

            return array("ID" => $contact->getId());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function createResource($externalId, $type, $isUser, $status, $login, $password, $name, $description,
                                   $class = null, $nickName = null, $documentNumber = null, $phone = null,
                                   $address = null, $district = null, $city = null, $zipCode = null, $parent = null,
                                   $availableFrom = null, $rating = 1)
    {
        $entity = $this->resourceFactory->createResource($externalId, $type, $isUser, $status, $login, $password, $name, $description, $class, $nickName,
            $documentNumber, $phone, $address, $district, $city, $zipCode, $parent,$availableFrom, $rating);
        $this->entityManager->persist($entity);
        return $entity;
    }

    public function updateResource($resource, $type, $isUser, $status, $login, $name, $description, $class = null, $nickName = null,
                                   $documentNumber = null, $phone = null, $address = null, $district = null, $city = null, $zipCode = null,
                                   $parent = null, $creator = null, $availableFrom = null, $rating = 1)
    {
        $resource = $this->resourceRepository->find($resource);
        if ($city) {
            $city = $this->cityRepository->find($city);
        }
        $resource = $this->resourceFactory->updateResource($resource, $type, $isUser, $status, $login, $name, $description, $class, $nickName, $documentNumber,
            $phone, $address, $district, $city, $zipCode, $parent, $creator,$availableFrom, $rating);

        $this->entityManager->persist($resource);
        return $resource;
    }

    public function createNewContact($sponsorId, $address, $city, $district, $email, $name, $phone = null, $status = null, $zipCode = null,$externalCode = null)
    {
        try{
            if($email){
                $isContact = $this->resourceRepository->findOneBy(array('email' => $email));
                if(!empty($isContact)){
                    throw new \Exception("Email já cadastrado. Favor informar outro email.");
                }
            }
            $contact = $this->createContact($sponsorId, $address, $city, $district, $email, $name, $phone, $status, $zipCode, $externalCode);
            return array("ID" => $contact->getId());
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function createContact($sponsorId, $address, $city, $district, $email, $name, $phone = null,
                                  $status = null, $zipCode = null,$externalCode = null)
    {
        $sponsor = $this->resourceRepository->find($sponsorId);
        if ($city) {
            $city = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\City', $city);
        }
        $contact = $this->createResource(null,self::CONTACT_RESOURCE, self::IS_USER_FALSE, $status, $email, null,
            $name, null, self::CLASS_CONTACT, null, null, $phone, $address, $district, $city, $zipCode, $sponsor);
        $contact->getExternalCode($externalCode);
        $this->entityManager->persist($contact);
        $this->entityManager->flush();

        return $contact;
    }

    public function deleteSponsor($sponsorId)
    {
        /**
         * @var \Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource $resource
         */
        $resource = $this->entityManager->find('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource', $sponsorId);
        $parent = $resource->getParent();
        $resource->setStatus('D');
        $this->entityManager->persist($resource);
        $this->entityManager->flush();
        return $parent;
    }

    public function editSponsor($login, $password, $status, $name, $nickName, $documentNumber, $description, $resourceId)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if($login){
                $sameEmailResource = $this->resourceRepository->findOneBy(array('email' => $login));
                if(!empty($sameEmailResource) && $sameEmailResource->getId() != $resourceId){
                    throw new \Exception("Email já cadastrado. Favor informar outro email.");
                }
            }

            if (empty($password) || empty($login)) {
                $isUser = self::IS_USER_FALSE;
            } else {
                $isUser = self::IS_USER_TRUE;
            }
            $resource = $this->entityManager->find(Utilities::ENTITY_DIR . 'Resource', $resourceId);
            $resource = $this->updateResource($resource, self::WORK_RESOURCE, $isUser, $status, $login, $name, $description, self::CLASS_SPONSOR, $nickName, $documentNumber);
            $resource = $this->resourceFactory->updateResourceTypeUser($resource, $login, $password, $status, $name);
            $this->entityManager->persist($resource);
            $this->entityManager->flush();
            $connection->commit();
            return $resource->getId();

        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
    }

    public function newSponsor($login, $password, $status, $name, $nickName, $documentNumber, $description)
    {
        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            if($login){
                $validateEmail = $this->resourceRepository->findOneBy(array('email' => $login));
                if(!empty($validateEmail)){
                    throw new \Exception("Email já cadastrado. Favor informar outro email.");
                }
            }
            $validationName = $this->entityManager->getRepository('\Zeedhi\DevOps\Model\Projectmanagement\Entities\Resource')->findOneBy(array('name' => $name, 'class' => self::IS_USER_TRUE));
            if (!empty($validationName)) {
                throw new \Exception("Já existe um cliente com este nome. Insira outro, por favor.");
            } else {
                if (empty($password) || empty($login)) {
                    $isUser = self::IS_USER_FALSE;
                } else {
                    $isUser = self::IS_USER_TRUE;
                }
                $resource = $this->createResource(null, self::WORK_RESOURCE, $isUser, $status, $login, $password, $name, $description, self::CLASS_SPONSOR, $nickName, $documentNumber);

                $this->entityManager->flush();
                $connection->commit();
            }
        } catch (\Exception $e) {
            $connection->rollback();
            throw $e;
        }
        return $resource;
    }

    public function uploadProfilePicture($imagePath)
    {
        $connection = $this->entityManager->getConnection();
        try{
            $connection->beginTransaction();
            $cdOperador = $this->environment->getCurrentUser()->getExternalCode();
            $operatorRepository = $this->entityManager->getRepository(self::TEKNISA_ENTITY . 'Operador');
            $operator = $operatorRepository->findOneBy(array('cdoperador' => $cdOperador));
            $odlImage = $operator->getDsimagemoper();
            $operator->setDsimagemoper($imagePath);
            $this->entityManager->persist($operator);
            $this->entityManager->flush();
            if($odlImage){
                $this->deleteProfilePictureInFileServe($odlImage);
            }
            $connection->commit();
            return [array('imagePath' => $imagePath)];
        } catch (\Exception $e) {
            $connection->rollBack();
            throw $e;
        }
    }

    public function deleteProfilePicture()
    {
        try{
            return $this->uploadProfilePicture(null);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function deleteProfilePictureInFileServe($imagePath)
    {
        $this->zhFileUploader->deleteFile($imagePath);
    }
}
